
/* Resume after suspension */

#include "sssd.h"
#include "internals.h"

#ifdef _INCLUDE_ESD
  #include <esd.h>
#endif

INLINE_PROTO int Log2(int value);

int sssd_resume(struct sssdaudio *ga, const char* device_name) {

    int format, stereo, speed, fragsize, skipconfig=0;

    /* Setup audio device */
    /* Setup audio device */
    if(device_name==DEFAULT_DEVICE) {
       ga->audio_fd = open( DEFAULT_DEVICE_NAME, O_WRONLY, 0 );
    } else
    #ifdef _INCLUDE_ESD
    if(strcmp(device_name,ESD_DEVICE_NAME)==0) {
       /* Setup ESD format parameters */
       
       if(ga->bits==16) format=ESD_BITS16;
       else format=ESD_BITS8;
       
       if(ga->channels==2) format|=ESD_STEREO;
       else format|=ESD_MONO;
       
       format|=ESD_PLAY | ESD_STREAM;
       
       ga->audio_fd = esd_play_stream( format, ga->freq, NULL, NULL);
       skipconfig=1;
       if(ga->audio_fd!=-1) { 
          format=ga->audioBufferSize*ga->numAudioBuffers;
          setsockopt( ga->audio_fd, SOL_SOCKET, SO_SNDBUF, &format, sizeof( format ) );
       }	  

    } else /* end ESD */
    #endif
    { 
       ga->audio_fd = open( device_name, O_WRONLY, 0 );
    }
       
    if(ga->audio_fd== -1) {
    /* Opening default device failed */  
       #ifndef __NOPRINTING__
       perror( device_name );
       #endif
       ga->lastERR=ERR_DEVICE_RESUME;
       return( ERR_DEVICE_RESUME );
    }

    /* Setup device parameters */
    if(ga->bits==16) {
       format = AFMT_S16_LE;
    } else {
       format = AFMT_U8;
    }
    
    stereo   = 0;
    if(ga->channels!=1) stereo = 1;
    speed    = ga->freq;

    if(skipconfig) goto postOSSconfig;


    /* Program audio device with parameters */
    if(ga->audioBufferSize!=0) {
       fragsize=Log2(ga->audioBufferSize)|0x00040000;
#ifndef __NOPRINTING__
       printf("Requesting audio buffer block size of %d\n",ga->audioBufferSize);
#endif
       if( ioctl(ga->audio_fd, SNDCTL_DSP_SETFRAGMENT, &fragsize) == -1 )
       {
#ifndef __NOPRINTING__
          perror("SNDCTL_DSP_SETFRAGMENT");  
#endif
          ga->lastERR=ERR_HARDWARE_PARMS;
	  goto err;
       }
    }

    if( ioctl(ga->audio_fd, SNDCTL_DSP_SETFMT, &format) == -1 )
    {
#ifndef __NOPRINTING__
        perror("SNDCTL_DSP_SETFMT");  
#endif
        ga->lastERR=ERR_HARDWARE_BITS;
        goto err;
    }
    if( ioctl(ga->audio_fd, SNDCTL_DSP_STEREO, &stereo) == -1 )
    {
#ifndef __NOPRINTING__
        perror("SNDCTL_DSP_STEREO");  
#endif
        ga->lastERR=ERR_HARDWARE_CHAN;
        goto err;
    }
    if( ioctl(ga->audio_fd, SNDCTL_DSP_SPEED, &speed) == -1 )
    {
#ifndef __NOPRINTING__
        perror("SNDCTL_DSP_SPEED");  
#endif
        ga->lastERR=ERR_HARDWARE_FREQ;
        goto err;
    }

    if( ioctl(ga->audio_fd, SNDCTL_DSP_GETBLKSIZE, &fragsize) == -1 )
    {
#ifndef __NOPRINTING__
        perror("SNDCTL_DSP_GETBLKSIZE");  
#endif
        ga->lastERR=ERR_HARDWARE_PARMS;
        goto err;
    }
    ga->audioBufferSize=fragsize;  

postOSSconfig:  /* place for ESD and aRts to go */

    sssd_unpauseAudio(ga);

    return 0;


err:

    close( ga->audio_fd );
    ga->audio_fd = -1;
    return(ga->lastERR);  
}

