
/* The code in this file is public domain  */
/* Author: Patrick Roberts */


/* Load a simple WAV format audio file - to replace loadSample removed
     from libsssd in libsssd demos.
     
*/

#include <stdio.h>
#include <stdlib.h>
#include "loadSimpleWave.h"

waveFile *loadSimpleWave(char *filename) {


   FILE *fp;
   void *data;
   int fileSize;
   waveFile *waveData;
   int i1, *ip1;
   
   fp=fopen(filename,"rb");
   if(fp==NULL) return NULL;

   #ifdef _DEBUG_
     printf("loadSimpleWave: %s opened\n",filename);
   #endif
   
   fseek(fp,0,SEEK_END);
   fileSize=ftell(fp);
   fseek(fp,0,SEEK_SET); /* no 'rewind' in WinCE */

   #ifdef _DEBUG_
     printf("loadSimpleWave: filesize=%d\n",fileSize);
   #endif
   
   data=malloc(fileSize);
   if(data==NULL) {
     fclose(fp);
     return NULL;
   }
   
   #ifdef _DEBUG_
     printf("loadSimpleWave: %d bytes allocated\n",fileSize);
   #endif

   fread(data,fileSize,1,fp);
   fclose(fp);

   #ifdef _DEBUG_
     printf("loadSimpleWave: file %s read into memory \n",filename);
   #endif

   waveData=(waveFile *)data;
   
   #ifdef _DEBUG_
     printf("loadSimpleWave: file info...\n\t"
	    "RIFF   =0x%x\n\t"
            "chunkID=0x%x\n\t"
	    "format =%d\n\t"
	    "chnls  =%d\n\t"
	    "freq   =%d\n\t"
	    "bits   =%d\n\t",
	    waveData->RIFF_dsize,
	    waveData->chunkID,
	    waveData->format,
	    waveData->channels,
	    waveData->freq,
	    waveData->bits);
   #endif

   if(waveData->chunkID!=0x20746d66) {
     free(data);
     return NULL;
   }
   
   if(waveData->format!=1) {
     free(data);
     return NULL;
   }
   
   #ifdef _DEBUG_
      puts("loadSimpleWave: Finding data chunk");
   #endif
   
   /* find data chunk*/
   i1=waveData->chunkSize+20;  /* the next format */
   ip1=(int *)((char *)data+i1);
   
   while(*ip1!=0x61746164 && i1<fileSize-16) {
     #ifdef _DEBUG_
        printf("loadSimpleWave: Skipping chunk id 0x%x (0x%x bytes)\n",*ip1,ip1[1]);
     #endif
     i1+=(ip1[1]+8);
     ip1=(int *)((char *)data+i1);     
   }
   
   /* did we not find a data chunk? */
   if(i1>=fileSize+16) {
     free(data);
     return NULL;
   }
   
   waveData->PCMdata=&ip1[2];
   waveData->RIFF_dsize=ip1[1];
   waveData->chunkSize=waveData->RIFF_dsize/waveData->align;

   #ifdef _DEBUG_
      printf("loadSimpleWave: PCM data found: %d bytes/%d samples (0x%x)\n",waveData->RIFF_dsize,waveData->chunkSize,*(int*)waveData->PCMdata);
   #endif

return waveData;
   
}
